/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- 
 * gui-debug-report.c: Tools for submitting useful bug report information.
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  tuba@ximian.com
 *  vlad@ximian.com
 */

#include <stdlib.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include <glib.h>
#include <gnome-xml/tree.h>

#include "gui.h"
#include "gui-debug-report.h"
#include "gui-util.h"
#include "gui-prefs.h"
#include "gui-cache.h"

#include "rc-dep-or.h"

/* encode 72 characters per line on email */
#define CHARS_PER_LINE  72

#define RC_DEBUG_EMAIL_ADDRESS "rc-debug-report@ximian.com"
#define DEBUG_INFO_PATH_NAME   "Debug_Info/Name"
#define DEBUG_INFO_PATH_EMAIL  "Debug_Info/Email"

static xmlNode *rc_package_dep_or_slist_to_xml_node (RCPackageDepSList *dep);

typedef struct _RCDebugInfo RCDebugInfo;
struct _RCDebugInfo
{
    gboolean abort;
    gchar *abort_reason;
    gchar *name;
    gchar *send_from_email;
    gchar *send_to_email;
    gchar *debug_filename;
    gchar *debug_file_path;
    gchar *error_string;
    gchar *error_details;
    gchar *debug_report_error;
    gchar *sendmail_path;
    time_t timestamp;
    
    FILE *sendmail_pipe;
    
    xmlDocPtr xml_doc;

    RCPackageSList *system_packages;
    RCPackageSList *packages_requested;
    RCResolver *resolver;

    GUIChannelSList *channel_list;
    GUIChannel *current_channel; 

    GnomeDialog  *dialog;
    GtkWidget *name_entry;
    GtkWidget *email_entry;
    GtkWidget *progress_bar;
};


static xmlNode *
rc_package_update_to_xml_node (RCPackageUpdate *update)
{
    xmlNode *update_node;
    gchar *tmp_string;

    update_node = xmlNewNode (NULL, "update");

    if (update->spec.epoch) {
        tmp_string = g_strdup_printf("%d", update->spec.epoch);
        xmlNewTextChild (update_node, NULL, "epoch", tmp_string);
        g_free(tmp_string);
    }

    xmlNewTextChild (update_node, NULL, "version", update->spec.version);

    if (update->spec.release) {
        xmlNewTextChild (update_node, NULL, "release", update->spec.release);
    }

    xmlNewTextChild (update_node, NULL, "filename",
                     g_basename (update->package_url));

    tmp_string = g_strdup_printf ("%d", update->package_size);
    xmlNewTextChild (update_node, NULL, "filesize", tmp_string);
    g_free (tmp_string);

    tmp_string = g_strdup_printf ("%d", update->installed_size);
    xmlNewTextChild (update_node, NULL, "installedsize", tmp_string);
    g_free (tmp_string);

    if (update->signature_url) {
        xmlNewTextChild (update_node, NULL, "signaturename",
                         update->signature_url);

        tmp_string = g_strdup_printf ("%d", update->signature_size);
        xmlNewTextChild (update_node, NULL, "signaturesize", tmp_string);
        g_free (tmp_string);
    }

    if (update->md5sum) {
        xmlNewTextChild (update_node, NULL, "md5sum", update->md5sum);
    }

    xmlNewTextChild (update_node, NULL, "importance",
                     rc_package_importance_to_string (update->importance));

    xmlNewTextChild (update_node, NULL, "description", update->description);

    return (update_node);
}

static xmlNode *
rc_package_dep_to_xml_node (RCPackageDep *dep_item)
{
    xmlNode *dep_node;

    if (dep_item->is_or) {
        RCPackageDepSList *dep_or_slist;
        dep_or_slist = rc_dep_string_to_or_dep_slist (dep_item->spec.name);
        dep_node = rc_package_dep_or_slist_to_xml_node (dep_or_slist);
        rc_package_dep_slist_free (dep_or_slist);
        return dep_node;
    }

    dep_node = xmlNewNode (NULL, "dep");

    xmlSetProp (dep_node, "name", dep_item->spec.name);

    if (dep_item->relation != RC_RELATION_ANY) {
        xmlSetProp (dep_node, "op",
                    rc_package_relation_to_string (dep_item->relation, FALSE));

        if (dep_item->spec.epoch > 0) {
            gchar *tmp;

            tmp = g_strdup_printf ("%d", dep_item->spec.epoch);
            xmlSetProp (dep_node, "epoch", tmp);
            g_free (tmp);
        }

        if (dep_item->spec.version) {
            xmlSetProp (dep_node, "version", dep_item->spec.version);
        }

        if (dep_item->spec.release) {
            xmlSetProp (dep_node, "release", dep_item->spec.release);
        }
    }

    return (dep_node);
}

static xmlNode *
rc_package_dep_or_slist_to_xml_node (RCPackageDepSList *dep)
{
    xmlNode *or_node;
    const RCPackageDepSList *dep_iter;

    or_node = xmlNewNode (NULL, "or");

    dep_iter = dep;
    while (dep_iter) {
        RCPackageDep *dep_item = (RCPackageDep *)(dep_iter->data);
        xmlAddChild (or_node, rc_package_dep_to_xml_node (dep_item));
        dep_iter = dep_iter->next;
    }

    return or_node;
}

static xmlNode *
rc_package_to_xml_node (RCPackage *package)
{
    xmlNode *package_node;
    xmlNode *tmp_node;
    RCPackageUpdateSList *history_iter;
    RCPackageDepSList *dep_iter;

    package_node = xmlNewNode (NULL, "package");

    xmlNewTextChild (package_node, NULL, "name", package->spec.name);
    xmlNewTextChild (package_node, NULL, "summary", package->summary);
    xmlNewTextChild (package_node, NULL, "description", package->description);
    xmlNewTextChild (package_node, NULL, "section",
                     rc_package_section_to_string (package->section));

    if (package->channel) {
        char *tmp = g_strdup_printf("%d", package->channel->id);

        xmlNewTextChild (package_node, NULL, "channel", tmp);

        g_free(tmp);
    }

    if (package->history) {
        tmp_node = xmlNewChild (package_node, NULL, "history", NULL);
        for (history_iter = package->history; history_iter;
             history_iter = history_iter->next)
        {
            RCPackageUpdate *update = (RCPackageUpdate *)(history_iter->data);
            xmlAddChild (tmp_node, rc_package_update_to_xml_node (update));
        }
    }

    if (package->requires) {
        tmp_node = xmlNewChild (package_node, NULL, "requires", NULL);
        for (dep_iter = package->requires; dep_iter;
             dep_iter = dep_iter->next)
        {
            RCPackageDep *dep = (RCPackageDep *)(dep_iter->data);

            xmlAddChild (tmp_node, rc_package_dep_to_xml_node (dep));
        }
    }

    if (package->recommends) {
        tmp_node = xmlNewChild (package_node, NULL, "recommends", NULL);
        for (dep_iter = package->recommends; dep_iter;
             dep_iter = dep_iter->next)
        {
            RCPackageDep *dep = (RCPackageDep *)(dep_iter->data);

            xmlAddChild (tmp_node, rc_package_dep_to_xml_node (dep));
        }
    }

    if (package->suggests) {
        tmp_node = xmlNewChild (package_node, NULL, "suggests", NULL);
        for (dep_iter = package->suggests; dep_iter;
             dep_iter = dep_iter->next)
        {
            RCPackageDep *dep = (RCPackageDep *)(dep_iter->data);

            xmlAddChild (tmp_node, rc_package_dep_to_xml_node (dep));
        }
    }

    if (package->conflicts) {
        tmp_node = xmlNewChild (package_node, NULL, "conflicts", NULL);
        for (dep_iter = package->conflicts; dep_iter;
             dep_iter = dep_iter->next)
        {
            RCPackageDep *dep = (RCPackageDep *)(dep_iter->data);

            xmlAddChild (tmp_node, rc_package_dep_to_xml_node (dep));
        }
    }

    if (package->provides) {
        tmp_node = xmlNewChild (package_node, NULL, "provides", NULL);
        for (dep_iter = package->provides; dep_iter;
             dep_iter = dep_iter->next)
        {
            RCPackageDep *dep = (RCPackageDep *)(dep_iter->data);

            xmlAddChild (tmp_node, rc_package_dep_to_xml_node (dep));
        }
    }

    return (package_node);
}

static gint
gui_debug_report_error_gui(char *warning)
{
    GnomeDialog *error_dialog;
    GtkWindow *main_window;

    main_window = GTK_WINDOW (GET_WIDGET("main_window"));
    error_dialog = GNOME_DIALOG (gnome_error_dialog_parented  (warning, main_window));
    gnome_dialog_run_and_close (error_dialog);
}


static char*
gui_debug_report_b64enc(const char *chunk, int size)
{
    static const char* to_b64 =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    int div = size / 3;
    int rem = size % 3;
    int chars = div*4 + rem + 1;
    int newlines = (chars + CHARS_PER_LINE - 1) / CHARS_PER_LINE;
    const char *data = chunk;
    char *string = (char*) malloc(chars + newlines + 1);
    
    if (string) {
        register char* buf = string;
        
        chars = 0;

        /*@+charindex@*/
        while (div > 0)
        {
            buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
            buf[1] = to_b64[((data[0] << 4) & 0x30) + ((data[1] >> 4) & 0xf)];
            buf[2] = to_b64[((data[1] << 2) & 0x3c) + ((data[2] >> 6) & 0x3)];
            buf[3] = to_b64[  data[2] & 0x3f];
            data += 3;
            buf += 4;
            div--;
            chars += 4;
            if (chars == CHARS_PER_LINE) {
                chars = 0;
                *(buf++) = '\n';
            }
        }
        
        switch (rem)
        {
        case 2:
            buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
            buf[1] = to_b64[((data[0] << 4) & 0x30) + ((data[1] >> 4) & 0xf)];
            buf[2] = to_b64[ (data[1] << 2) & 0x3c];
            buf[3] = '=';
            buf += 4;
            chars += 4;
            break;
        case 1:
            buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
            buf[1] = to_b64[ (data[0] << 4) & 0x30];
            buf[2] = '=';
            buf[3] = '=';
            buf += 4;
            chars += 4;
            break;
        }
        /*@=charindex@*/
        
        /*  *(buf++) = '\n'; This would result in a buffer overrun */
        *buf = '\0';
    }
    
    return string;
}


static gint
gui_debug_report_calc_size (gint size)
{
    gint new_size;
    int div, rem, chars, newlines;
    
    div = size / 3;
    rem = size % 3;
    chars = div*4 + rem + 1;
    newlines = (chars + CHARS_PER_LINE - 1) / CHARS_PER_LINE;
    new_size = chars + newlines + 1;

    return new_size;
}

static void
gui_debug_report_send_mail (RCDebugInfo *info, FILE *attachment, gint size)
{
	gint new_size;
	gchar *message,*string2; 
    int bytes_to_write, bytes_writen;

    g_return_if_fail (attachment != NULL);
    g_return_if_fail (size != 0);

    info->sendmail_pipe = popen (g_strdup_printf ("%s -t",info->sendmail_path), "w");
	fprintf(info->sendmail_pipe,
            "From: %s\n"
            "To: %s\n"
            "Subject: Debug report: %s\n"
            "MIME-Version: 1.0\n"
            "Content-Type: application/x-gzip; name=%s\n"
            "Content-Transfer-Encoding: base64\n"
            "Content-Disposition: attachment\n",
            info->send_from_email,
            info->send_to_email,
            info->error_string,
            info->debug_filename);

    new_size = gui_debug_report_calc_size (size);

	message = (char*) malloc(size);
    fread(message,sizeof(char),size,attachment);
    string2 = (char*) malloc(new_size);
	string2 =  gui_debug_report_b64enc(message, size);

    bytes_to_write = new_size;
    bytes_writen = 0;
    
    /* send the mail in blocks to fill the progress bar */
    while (bytes_to_write>0) {
        gfloat percentage;
        gint write;
        gint bytes_writen_now;

        write = (bytes_to_write > 1024) ? 1024 : bytes_to_write;
        bytes_writen_now = fwrite(string2 + bytes_writen,sizeof(char),write,info->sendmail_pipe);
        bytes_writen   += bytes_writen_now;
        bytes_to_write -= bytes_writen_now;

        percentage = ((gfloat)bytes_writen) / ((gfloat)new_size);
        gtk_progress_set_percentage (GTK_PROGRESS (info->progress_bar),
                                     percentage );
        while (gtk_events_pending ())
            gtk_main_iteration ();
    }

	fflush(info->sendmail_pipe);
	pclose(info->sendmail_pipe);
}


static void
gui_debug_report_add_package_list_xml (RCDebugInfo *info, char *node_name, RCPackageSList *list)
{
    RCPackageSList *iter;
    xmlNode *np;
   
    np = xmlNewNode (NULL, node_name);
    iter = list;
    while (iter) {
        RCPackage *pkg = (RCPackage *) iter->data;
        xmlAddChild (np, rc_package_to_xml_node (pkg));
        iter = iter->next;
    }
    xmlAddChild (info->xml_doc->root, np);
   
}

/* FIXME: This stuff is really not ideal, but will have to do until the
 * new dependency code matures enough and contains better detailed logging.
 */
static void
output_cb (RCResolverInfo *info, gpointer user_data)
{
    xmlNode *np = user_data;
    char *msg = rc_resolver_info_to_str(info);

    xmlNewTextChild (np, NULL, "msg", msg);

    g_free(msg);
}

static void
gui_debug_report_add_dependency_spew (RCDebugInfo *info)
{
    xmlNode *np;
    RCResolverQueue *queue;

    /* FIXME: This sucks, but is techinically valid */
    queue = (RCResolverQueue *) info->resolver->invalid_queues->data;

    np = xmlNewNode (NULL, "dependency_debug");
    rc_resolver_context_foreach_info(queue->context, NULL, -1, output_cb, np);

    xmlAddChild (info->xml_doc->root, np);
}

static void
gui_debug_report_add_channel_xml (RCDebugInfo *info)
{
    xmlNode *np;
    gchar *idstr = g_strdup_printf ("%d", info->current_channel->rcchannel.id);

    np = xmlNewNode (NULL, "current_channel");
    xmlSetProp (np, "id", idstr);
    xmlSetProp (np, "name", info->current_channel->rcchannel.name);
    g_free (idstr);

    xmlAddChild (info->xml_doc->root, np);

}

static void
gui_debug_report_add_channel_list_xml (RCDebugInfo *info)
{
    xmlNode *np;
    GUIChannelSList *iter = info->channel_list;

    np = xmlNewNode (NULL, "channels");

    while (iter) {
        GUIChannel *gch = (GUIChannel *) iter->data;
        gchar *idstr = g_strdup_printf ("%d", gch->rcchannel.id);
        xmlNode *chnp;

        chnp = xmlNewNode (NULL, "channel");
        xmlSetProp (chnp, "subscribed", gch->subscribed ? "1" : "0");
        xmlSetProp (chnp, "id", idstr);
        xmlSetProp (chnp, "name", gch->rcchannel.name);
        g_free (idstr);
        xmlAddChild (np, chnp);

        iter = iter->next;
    }

    xmlAddChild (info->xml_doc->root, np);
}

    
static void
gui_debug_report_add_error_xml (RCDebugInfo *info)
{
    xmlNode *np;

    np = xmlNewTextChild(
        info->xml_doc->root, NULL, "error", info->error_details);
    xmlSetProp(np, "type", info->error_string);
}

static void
gui_debug_report_create_xml_doc (RCDebugInfo *info)
{
    char *tmp;

    info->xml_doc = xmlNewDoc ("1.0");
    info->xml_doc->root = xmlNewNode (NULL, "rc_bug_report");

    xmlSetProp (info->xml_doc->root, "version", VERSION);
    xmlSetProp (info->xml_doc->root, "build", BUILD);
    xmlSetProp (info->xml_doc->root, "distro", rc_distro_get_name());
    xmlSetProp (info->xml_doc->root, "reporter_name", info->name);
    xmlSetProp (info->xml_doc->root, "reporter_email", info->send_from_email);

    tmp = g_strdup_printf("%ld", info->timestamp);
    xmlSetProp (info->xml_doc->root, "timestamp", tmp);
    g_free(tmp);
}


static void
gui_debug_report_save_xml_doc (RCDebugInfo *info)
{
    char *file = g_strdup_printf(
        "%s/%s", info->debug_file_path, info->debug_filename);

    xmlSetDocCompressMode (info->xml_doc, 9);
    xmlSaveFile (file, info->xml_doc);
    xmlFreeDoc (info->xml_doc);
    g_free(file);
}


static void
gui_debug_report_create_xml (RCDebugInfo *info)
{
    gui_debug_report_create_xml_doc (info);

    gui_debug_report_add_error_xml(info);

    if (info->packages_requested)
        gui_debug_report_add_package_list_xml (info, "requested_packages", info->packages_requested);
    
    if (!info->resolver->best_context)
        gui_debug_report_add_dependency_spew (info);

    if (info->current_channel)
        gui_debug_report_add_channel_xml (info);

    if (info->channel_list)
        gui_debug_report_add_channel_list_xml (info);

    if (info->resolver->packages_to_install)
        gui_debug_report_add_package_list_xml (info, "packages_to_install", info->resolver->packages_to_install);

    if (info->resolver->packages_to_remove)
        gui_debug_report_add_package_list_xml (info, "packages_to_remove", info->resolver->packages_to_remove);

    if (info->system_packages)
        gui_debug_report_add_package_list_xml (info, "system_packages", info->system_packages);
    
    gui_debug_report_save_xml_doc (info);
}

static void
gui_debug_report_get_dep_resolution (RCDebugInfo *info)
{
    RCResolver *resolver;

    resolver = rc_resolver_new();
    rc_resolver_add_packages_to_install_from_slist(
        resolver, info->packages_requested);
    rc_resolver_resolve_dependencies(resolver);

    info->resolver = resolver;
}

static gboolean
gui_debug_report_populate_info (RCDebugInfo *info, const char *error)
{
    RCPackageSList **req = NULL;

    info->dialog = GNOME_DIALOG(GET_WIDGET ("debug_dialog"));
    info->name_entry   = GET_WIDGET ("rc_debug_name");
    info->email_entry  = GET_WIDGET ("rc_debug_email");
    info->progress_bar = GET_WIDGET ("rc_debug_progress");
    
    if ((!info->progress_bar) || (!info->dialog) || (!info->name_entry) || (!info->email_entry)) {
        info->debug_report_error = "Check your glade file";
        gui_debug_report_error_gui(info->debug_report_error);
        return FALSE;
    }
    
    gtk_progress_set_percentage (GTK_PROGRESS (info->progress_bar), 0.0 );

    info->send_to_email = getenv ("RC_DEBUG_EMAIL");
    if (!info->send_to_email)
        info->send_to_email = RC_DEBUG_EMAIL_ADDRESS;

    info->timestamp = time(NULL);

    info->name                      = gui_prefs_get_string (DEBUG_INFO_PATH_NAME);
    info->send_from_email           = gui_prefs_get_string (DEBUG_INFO_PATH_EMAIL);
    info->debug_file_path           = gui_cache_get_cache_directory(appinfo->normal_cache);

    info->debug_filename = g_strdup_printf(
        "rc-debug-report-%ld.gz", info->timestamp);

    info->system_packages = appinfo->system_packages;
    info->current_channel = appinfo->current_channel;
    info->channel_list    = appinfo->channel_list;
    
    info->sendmail_path = gnome_is_program_in_path ("sendmail");
    if (!info->sendmail_path) {
        if (g_file_exists ("/usr/sbin/sendmail"))
            info->sendmail_path = g_strdup ("/usr/sbin/sendmail");
        else if (g_file_exists ("/usr/lib/sendmail"))
            info->sendmail_path = g_strdup ("/usr/lib/sendmail");
        else {
            info->debug_report_error = "Unable to fork sendmail";
            gui_debug_report_error_gui(info->debug_report_error);
            return FALSE;
        }
    }
   
    switch (appinfo->current_page) {
    case PAGE_AVAILABLE:
        req = (RCPackageSList **) gtk_object_get_data(
            GTK_OBJECT(appinfo->available_html), "packages_to_install");
        break;
    case PAGE_UPDATES:
        req = (RCPackageSList **) gtk_object_get_data(
            GTK_OBJECT(appinfo->update_html), "packages_to_install");
        break;
    case PAGE_INSTALLED:
        req = (RCPackageSList **) gtk_object_get_data(
            GTK_OBJECT(appinfo->installed_html), "packages_to_remove");
        break;
    default:
        g_assert_not_reached();
        break;
    }

    /* Special case for local installs.  Very ugly.  See 
     * gui-util.c:install_local_files() and cry.
     */
    if (!req) {
        info->packages_requested = gtk_object_get_data(
            GTK_OBJECT(appinfo->update_html), "local_installs");
    }
    else {
        info->packages_requested = *req;
    }

    info->error_string = g_strdup(error);

    if (rc_packman_get_error(appinfo->pman))
        info->error_details = g_strdup(rc_packman_get_reason(appinfo->pman));

    gui_debug_report_get_dep_resolution (info);    
  
    return TRUE;
}

static void
gui_debug_report_free_info (RCDebugInfo *info)
{
    if (info->resolver)
        rc_resolver_free(info->resolver);
    if (info->debug_filename)
        g_free (info->debug_filename);
    if (info->debug_file_path)
        g_free (info->debug_file_path);
    if (info->error_string)
        g_free (info->error_string);
    if (info->error_details)
        g_free (info->error_details);
    
    info->system_packages     = NULL;
    info->packages_requested  = NULL;
    info->resolver            = NULL;
    info->debug_filename      = NULL;
    info->debug_file_path     = NULL;
    info->error_string        = NULL;
    info->error_details       = NULL;
    info->current_channel     = NULL;
    info->channel_list        = NULL;
    
    g_free (info);    
}

static gint
gui_debug_report_query_user (RCDebugInfo *info)
{
    gint ret;

    gnome_dialog_close_hides(info->dialog, TRUE);

    /* Sensitize the widgets */
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_ok"), TRUE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_cancel"), TRUE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_name"), TRUE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_email"), TRUE);

    if (info->name)
        gtk_entry_set_text(GTK_ENTRY(info->name_entry), info->name);
    if (info->send_from_email)
        gtk_entry_set_text(GTK_ENTRY(info->email_entry), info->send_from_email );

    ret = gnome_dialog_run (info->dialog);
   
    info->name = gtk_entry_get_text (GTK_ENTRY (info->name_entry));
    info->send_from_email = gtk_entry_get_text (GTK_ENTRY (info->email_entry));

    gui_prefs_set_string(DEBUG_INFO_PATH_NAME,  info->name);
    gui_prefs_set_string(DEBUG_INFO_PATH_EMAIL, info->send_from_email);
   
    return (!ret); /* ret contains 0 if user clicked ok */
}


static void
gui_debug_report_send_report (RCDebugInfo *info)
{
    char *file;
    struct stat s;
    gint size;                     
    FILE *tmpfp;

    /* Desensitize the widgets */
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_ok"), FALSE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_cancel"), FALSE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_name"), FALSE);
    gtk_widget_set_sensitive(GET_WIDGET("rc_debug_email"), FALSE);

    file = g_strdup_printf(
        "%s/%s", info->debug_file_path, info->debug_filename);

    stat (file, &s);
    size = s.st_size;
    tmpfp = fopen (file, "r");
    if (tmpfp == NULL) {
        info->debug_report_error = g_strdup_printf ("Could not create %s", file);
        g_free(file);
        gui_debug_report_error_gui(info->debug_report_error);
        return;
    }

    g_free(file);
        
    gui_debug_report_send_mail (info, tmpfp, size);
    fclose (tmpfp);
}

void
gui_debug_report_main (const char *error)
{
    RCDebugInfo *info;

    info = g_new0 (RCDebugInfo,1);
    if (!gui_debug_report_populate_info (info, error)) {
        return;
    }
    if (!gui_debug_report_query_user(info)) {
        gui_debug_report_free_info (info);
        gnome_dialog_close (info->dialog);
        return;
    }
    gui_debug_report_create_xml (info);
    gui_debug_report_send_report (info);
    gnome_dialog_close (info->dialog);
    
    gui_debug_report_free_info (info);
}
