/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui-prefs.c: proxy / mirror settings
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include "gui.h"
#include "gui-prefs.h"
#include "gui-priority.h"
#include "gui-transfer.h"
#include "mirror.h"

typedef struct _ConfVal ConfVal;

typedef void (*GUIPrefsFunc) (ConfVal *val);

struct _ConfVal {
    char *widget_name;
    char *conf_path;
    GUIPrefsFunc load_pref_func;
    GUIPrefsFunc save_pref_func;
    void *p;
    gboolean priv : 1;
    gpointer data;
    GtkWidget *w;
};

typedef struct {
    GtkWidget *dialog;
    int initializing;
    
    /* Proxy options */
    ProxyType proxy_type;
    gboolean proxy_auth;
    gchar *proxy_host;
    gchar *proxy_port;
    gchar *proxy_name;
    gchar *proxy_pass;
    gboolean http10_enabled;

    /* Cache options */
    gboolean cache_enabled;
    gchar *cache_path;
    gboolean cache_clear_on_startup;
    gboolean cache_clear_on_exit;
    gboolean cache_time_enabled;
    gchar *cache_time;

    /* Mirror options */
    gchar *mirror_url;
    gchar *priority_mirror_url;

    /* Signature options */
    gboolean sign_ask_enabled;

    /* GUI options */
    gboolean show_pretty_names;

    /* Priority Updates options */
    gboolean priority_enabled;
} RCConfig;

RCConfig rcconfig = { 0 };

GtkWidget *proxy_rbs[5] = { NULL };

static void load_toggle_func(ConfVal *val);
static void load_entry_func (ConfVal *val);
static void load_list_func  (ConfVal *val);
static void load_radio_func (ConfVal *val);

static void save_toggle_func(ConfVal *val);
static void save_entry_func (ConfVal *val);
static void save_list_func  (ConfVal *val);
static void save_radio_func (ConfVal *val);

static void load_mirror_list_func(ConfVal *val);
static void save_mirror_list_func(ConfVal *val);

static ConfVal conf[] = {
    { "no_proxy_rb", "Proxy/Type",
      load_radio_func,  save_radio_func,
      &rcconfig.proxy_type, FALSE, proxy_rbs },

    { "proxy_host",   "Proxy/Host",
      load_entry_func, save_entry_func,  
      &rcconfig.proxy_host },

    { "proxy_port",   "Proxy/Port",
      load_entry_func, save_entry_func, 
      &rcconfig.proxy_port },

    { "proxy_auth",   "Proxy/UseAuth",
      load_toggle_func, save_toggle_func,
      &rcconfig.proxy_auth },

    { "proxy_name",   "Proxy/Name",
      load_entry_func, save_entry_func, 
      &rcconfig.proxy_name },

    { "proxy_pass",   "Proxy/Pass",
      load_entry_func, save_entry_func, 
      &rcconfig.proxy_pass, TRUE },

    { "http10_enable","Proxy/HTTP10",
      load_toggle_func, save_toggle_func,
      &rcconfig.http10_enabled },

    { "cache_enable", "Cache/Enabled=TRUE",
      load_toggle_func, save_toggle_func, 
      &rcconfig.cache_enabled },

    { "cache_path",   NULL /* set in gui_prefs_load */,
      load_entry_func, save_entry_func, 
      &rcconfig.cache_path },

    { "cache_clear_packages_on_startup", "Cache/ClearOnStartup",
      load_toggle_func, save_toggle_func,
      &rcconfig.cache_clear_on_startup },

    { "cache_clear_packages_on_exit", "Cache/ClearOnExit",
      load_toggle_func, save_toggle_func,
      &rcconfig.cache_clear_on_exit },

    { "cache_clear_packages_over_time", "Cache/ExpireOverTime",
      load_toggle_func, save_toggle_func,
      &rcconfig.cache_time_enabled },

    { "cache_time",   "Cache/ExpireTime",
      load_entry_func, save_entry_func,
      &rcconfig.cache_time },

    { "mirror_clist", NULL,
      load_mirror_list_func, save_mirror_list_func,
      NULL },

    { "sign_ask_enable", "Signatures/Ask=TRUE",
      load_toggle_func, save_toggle_func,
      &rcconfig.sign_ask_enabled },

    { "gui_show_pretty_names", "GUI/PrettyNames=TRUE",
      load_toggle_func, save_toggle_func,
      &rcconfig.show_pretty_names },

    { "priority_enable", "Priority/Enabled=TRUE",
      load_toggle_func, save_toggle_func,
      &rcconfig.priority_enabled },

    { NULL }
};

static void apply_settings(void);
static const char *proxy_type_to_string(ProxyType proxy_type);

#define BEGIN_ROOT_MAGIC do { if (!appinfo->not_root) seteuid(appinfo->uid); gnome_config_push_prefix("/red-carpet/"); } while (0)
#define END_ROOT_MAGIC   do { if (!appinfo->not_root) seteuid(0); gnome_config_pop_prefix(); } while (0)

/* We have to wrap all of the gnome_config API to work with NFS home
 * directories.  Kill me.
 */

/* Getters */
static gboolean
rc_config_get_bool(char *path)
{
    gboolean value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_get_bool(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_get_bool */

static gboolean
rc_config_private_get_bool(char *path)
{
    gboolean value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_private_get_bool(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_private_get_bool */

static int
rc_config_get_int(char *path)
{
    int value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_get_int(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_get_int */

static int
rc_config_private_get_int(char *path)
{
    int value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_private_get_int(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_private_get_int */

static char *
rc_config_get_string(char *path)
{
    char *value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_get_string(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_get_string */

static char *
rc_config_private_get_string(char *path)
{
    char *value;

    BEGIN_ROOT_MAGIC;
    value = gnome_config_private_get_string(path);
    END_ROOT_MAGIC;

    return value;
} /* rc_config_private_get_string */

/* Setters */
static void
rc_config_set_bool(char *path, gboolean value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_set_bool(path, value);
    END_ROOT_MAGIC;
} /* rc_config_set_bool */

static void
rc_config_private_set_bool(char *path, gboolean value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_private_set_bool(path, value);
    END_ROOT_MAGIC;
} /* rc_config_private_set_bool */

static void
rc_config_set_int(char *path, int value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_set_int(path, value);
    END_ROOT_MAGIC;
} /* rc_config_set_booint */

static void
rc_config_private_set_int(char *path, int value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_private_set_int(path, value);
    END_ROOT_MAGIC;
} /* rc_config_private_set_int */

static void
rc_config_set_string(char *path, const char *value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_set_string(path, value);
    END_ROOT_MAGIC;
} /* rc_config_set_string */

static void
rc_config_private_set_string(char *path, const char *value)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_private_set_string(path, value);
    END_ROOT_MAGIC;
} /* rc_config_private_set_string */

static void
rc_config_clean_key(char *path)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_clean_key(path);
    END_ROOT_MAGIC;
} /* gnome_config_clean_key */

static void
rc_config_sync(void)
{
    BEGIN_ROOT_MAGIC;
    gnome_config_sync();
    END_ROOT_MAGIC;
} /* rc_config_sync */

/* To relieve some of the swelling... */
#define BOOL_P (*((gboolean *) val->p))
#define INT_P  (*((int *)      val->p))
#define CHAR_P (*((gchar **)   val->p))

static void
load_toggle_func(ConfVal *val)
{
    BOOL_P = val->priv ? 
        rc_config_private_get_bool(val->conf_path) :
        rc_config_get_bool(val->conf_path);

    if (!appinfo->not_root)
        seteuid(0);

    gtk_toggle_button_set_active(
        GTK_TOGGLE_BUTTON(val->w), BOOL_P);

    gtk_signal_emit_by_name(GTK_OBJECT(val->w), "toggled");
} /* load_toggle_func */

static void
save_toggle_func(ConfVal *val)
{
    BOOL_P = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(val->w));

    if (val->priv)
        rc_config_private_set_bool(val->conf_path, BOOL_P);
    else
        rc_config_set_bool(val->conf_path, BOOL_P);
} /* save_toggle_func */

static void
load_entry_func(ConfVal *val)
{
    g_free(CHAR_P);

    CHAR_P = val->priv ?
        rc_config_private_get_string(val->conf_path) :
        rc_config_get_string(val->conf_path);

    if (!appinfo->not_root)
        seteuid(0);
    
    if (CHAR_P)
        gtk_entry_set_text(GTK_ENTRY(val->w), CHAR_P);
} /* load_entry_func */

static void
save_entry_func(ConfVal *val)
{
    g_free(CHAR_P);
    CHAR_P = gtk_editable_get_chars(GTK_EDITABLE(val->w), 0, -1);

    if (val->priv)
        rc_config_private_set_string(val->conf_path, CHAR_P);
    else
        rc_config_set_string(val->conf_path, CHAR_P);
} /* save_entry_func */

static void
load_mirror_list_func(ConfVal *val)
{
    if (appinfo->priority) {
        val->conf_path = "Mirror/PriorityURL";
        val->p = &rcconfig.priority_mirror_url;
    }
    else {
        val->conf_path = "Mirror/URL";
        val->p = &rcconfig.mirror_url;
    }

    load_list_func(val);
} /* load_mirror_list_func */

static void
save_mirror_list_func(ConfVal *val)
{
    if (appinfo->priority) {
        val->conf_path = "Mirror/PriorityURL";
        val->p = &rcconfig.priority_mirror_url;
    }
    else {
        val->conf_path = "Mirror/URL";
        val->p = &rcconfig.mirror_url;
    }

    save_list_func(val);
} /* save_mirror_list_func */

static void
load_list_func(ConfVal *val)
{
    g_free(CHAR_P);

    CHAR_P = val->priv ?
        rc_config_private_get_string(val->conf_path) :
        rc_config_get_string(val->conf_path);

    if (CHAR_P) {
        int i;

        /* First off, if the string is empty (""), then we would prefer if it
           were NULL. Delight in the dereferencing pain! */
        if (!(*CHAR_P))
            CHAR_P = NULL;

        for (i = 0; i < GTK_CLIST(val->w)->rows; i++) {
            char *url = gtk_clist_get_row_data(GTK_CLIST(val->w), i);
            
            /* url and val->p are NULL. That's a match */
            if (!url && !CHAR_P)
                gtk_clist_select_row(GTK_CLIST(val->w), i, 0);
            else if (url && CHAR_P) {
                if (url && g_strcasecmp(CHAR_P, url) == 0)
                    gtk_clist_select_row(GTK_CLIST(val->w), i, 0);
            }
        }
    }
} /* load_list_func */

static void
save_list_func(ConfVal *val)
{
    int row = GPOINTER_TO_INT(
        gtk_object_get_data(GTK_OBJECT(val->w), "selected_row"));

    /* Nothing is set, so just leave it as is. */
    if (row == -1)
        return;

    g_free(CHAR_P);
    CHAR_P = g_strdup(gtk_clist_get_row_data(GTK_CLIST(val->w), row));

    if (val->priv)
        rc_config_private_set_string(val->conf_path, CHAR_P);
    else
        rc_config_set_string(val->conf_path, CHAR_P);
} /* save_list_func */

static void
load_radio_func(ConfVal *val)
{
    GtkWidget **rbs = val->data;

    INT_P = val->priv ?
        rc_config_private_get_int(val->conf_path) :
        rc_config_get_int(val->conf_path);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbs[INT_P]), TRUE);

    /* Sucks, but we need to make sure things are set up in the default case */
    gtk_signal_emit_by_name(GTK_OBJECT(rbs[INT_P]), "toggled");
} /* load_radio_func */

static void
save_radio_func(ConfVal *val)
{
    GtkWidget **rbs = val->data;
    int i;

    INT_P = -1;

    for (i = 0; rbs[i]; i++) {
        gboolean a;

        a = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rbs[i]));

        if (a) {
            INT_P = i;
            break;
        }
    }
    g_assert(INT_P != -1);

    if (val->priv)
        rc_config_private_set_int(val->conf_path, INT_P);
    else
        rc_config_set_int(val->conf_path, INT_P);
} /* save_radio_func */

void
gui_prefs_load(void)
{
    ConfVal *val;

    if (!rcconfig.dialog)
        rcconfig.dialog = GET_WIDGET ("property_dialog");

    /* Populate the proxy_rbs array. A little hackish, yes. */
    proxy_rbs[0] = GET_WIDGET("no_proxy_rb");
    proxy_rbs[1] = GET_WIDGET("http_proxy_rb");
    proxy_rbs[2] = GET_WIDGET("socks4_proxy_rb");
    proxy_rbs[3] = GET_WIDGET("socks5_proxy_rb");
    proxy_rbs[4] = NULL;

    rcconfig.initializing = TRUE;

    /* We need to set the mirror's selected_row thingy to -1, which means
       that no item is selected so that if a user has a mirror selected
       which is no longer in the list, it won't be changed for them if
       they visit the preferences dialog. */
    gtk_object_set_data(
        GTK_OBJECT(GET_WIDGET("mirror_clist")),
        "selected_row", GINT_TO_POINTER(-1));

    /* Set the default cache path depending on whether or not this is being
       run as root. */
    conf[8].conf_path = appinfo->not_root ? 
        g_strdup_printf("Cache/Path=%s/.redcarpet", getenv("HOME")) : 
        g_strdup("Cache/Path=/var/cache/redcarpet");

    for (val = conf; val->widget_name; val++) {
        val->w = GET_WIDGET (val->widget_name);
        (val->load_pref_func)(val);
    }

    /* Backward compatibility hack here... */
    if (rc_config_get_int("Proxy/Type=-1") == -1 &&
        rc_config_get_bool("Proxy/Enabled") == TRUE)
        rcconfig.proxy_type = PROXY_TYPE_HTTP;

    rcconfig.initializing = FALSE;

    apply_settings();

    g_free(conf[8].conf_path);
    conf[8].conf_path = "Cache/Path";
} /* gui_prefs_load */

void
gui_prefs_save(void)
{
    ConfVal *val;

    /* Remove the Proxy/Enabled key */
    rc_config_clean_key("Proxy/Enabled");

    for (val = conf; val->widget_name; val++)
        (val->save_pref_func)(val);

    rc_config_sync ();

    apply_settings();
} /* gui_prefs_save */

void
gui_prefs_dialog_show(void)
{
    GnomePropertyBox *box = GNOME_PROPERTY_BOX(rcconfig.dialog);

    gtk_widget_hide(box->help_button);

    gnome_dialog_close_hides(GNOME_DIALOG(rcconfig.dialog), TRUE);
    gnome_dialog_set_parent(
        GNOME_DIALOG(rcconfig.dialog), 
        GTK_WINDOW(GET_WIDGET("main_window")));

    gtk_window_set_title(
        GTK_WINDOW(rcconfig.dialog), _("Ximian Red Carpet - Preferences"));

    gtk_widget_show(rcconfig.dialog);
    gdk_window_raise(rcconfig.dialog->window);
} /* gui_prefs_dialog_show */

ProxyType
gui_prefs_get_proxy_type(void)
{
    return rcconfig.proxy_type;
} /* gui_prefs_get_proxy_type */

char *
gui_prefs_get_proxy_url(void)
{
    char *url;
    const char *method;
    char *auth = NULL;

    if (rcconfig.proxy_type == PROXY_TYPE_NONE)
        return NULL;

    method = proxy_type_to_string(rcconfig.proxy_type);

    if (rcconfig.proxy_auth) {
        char *p = NULL;
        
        if (rcconfig.proxy_pass && *rcconfig.proxy_pass)
            p = g_strdup_printf(":%s", rcconfig.proxy_pass);
        
        auth = g_strdup_printf("%s%s@", rcconfig.proxy_name, p ? p : "");
        g_free(p);
    }
    
    url = g_strdup_printf(
        "%s://%s%s:%s", method, auth ? auth : "",
        rcconfig.proxy_host, rcconfig.proxy_port);
    g_free(auth);

    return url;
} /* gui_prefs_get_proxy_url */

gboolean
gui_prefs_get_http10_enabled(void)
{
    return rcconfig.http10_enabled;
} /* gui_prefs_get_http10_enabled */

gboolean
gui_prefs_get_cache_enabled(void)
{
    return rcconfig.cache_enabled;
} /* gui_prefs_get_cache_enabled */

void
gui_prefs_get_cache_info(char **path, int *days,
                         gboolean *clear_on_startup, gboolean *clear_on_exit)
{
    if (!rcconfig.cache_enabled) {
        char *p;

        if (appinfo->not_root)
            p = g_strdup_printf("%s/.redcarpet", getenv("HOME"));
        else
            p = g_strdup("/var/cache/redcarpet");

        if (path)
            *path = p;

        if (days)
            *days = 0;

        if (clear_on_startup)
            *clear_on_startup = FALSE;

        if (clear_on_exit)
            *clear_on_exit = TRUE;
    }
    else {
        if (path)
            *path = g_strdup(rcconfig.cache_path);

        if (days) {
            if (rcconfig.cache_time_enabled)
                *days = atoi(rcconfig.cache_time);
            else
                *days = 0;
        }
        
        if (clear_on_startup)
            *clear_on_startup = rcconfig.cache_clear_on_startup;
        
        if (clear_on_exit)
            *clear_on_exit = rcconfig.cache_clear_on_exit;
    }
} /* gui_prefs_get_cache_info */

const char *
gui_prefs_get_mirror_url(void)
{
    char *url;

    if (appinfo->priority) {
        url = rcconfig.priority_mirror_url = rc_config_get_string(
            "Mirror/PriorityURL");
    }
    else {
        url = rcconfig.mirror_url = rc_config_get_string("Mirror/URL");
    }

    /* Workaround for files which somehow get empty URL or PriorityURL lines */
    if (url && !*url) {
        g_warning ("URL or PriorityURL has empty value!");
        if (appinfo->priority)
            url = rcconfig.priority_mirror_url = NULL;
        else
            url = rcconfig.mirror_url = NULL;
    }

    return url;
} /* gui_prefs_get_mirror_url */

void
gui_prefs_set_mirror_url(const char *url, PriorityType type)
{
    GtkCList *clist;
    int i;

    if (type == PRIORITY_TYPE_PRIORITY ||
        (type == PRIORITY_TYPE_CURRENT && appinfo->priority)) {
        g_free(rcconfig.priority_mirror_url);
        rcconfig.priority_mirror_url = g_strdup(url);
        rc_config_set_string("Mirror/PriorityURL", url);
    }
    else {
        g_free(rcconfig.mirror_url);
        rcconfig.mirror_url = g_strdup(url);
        rc_config_set_string("Mirror/URL", url);
    }

    rc_config_sync();

    clist = GTK_CLIST(GET_WIDGET("mirror_clist"));
    for (i = 0; i < clist->rows; i++) {
        char *u = gtk_clist_get_row_data(clist, i);

        if ((!u && !url) || (u && url && g_strcasecmp(u, url) == 0))
            gtk_clist_select_row(clist, i, 0);
    }
} /* gui_prefs_set_mirror_url */

gboolean
gui_prefs_get_signature_ask(void)
{
    return rcconfig.sign_ask_enabled;
} /* gui_prefs_get_signature_ask */

void
gui_prefs_set_signature_ask(gboolean ask)
{
    GtkWidget *button;

    rcconfig.sign_ask_enabled = ask;
    
    button = GET_WIDGET("sign_ask_enable");
    GTK_TOGGLE_BUTTON(button)->active = ask;
    gtk_widget_queue_draw(button);

    /* This probably isn't the right way to do it, but we're gonna save this
       element and just this element now. */
    rc_config_set_bool("Signatures/Ask", ask);
    rc_config_sync();
} /* gui_prefs_set_signature_ask */

gboolean
gui_prefs_get_pretty_names(void)
{
    return rcconfig.show_pretty_names;
} /* gui_prefs_get_pretty_names */

gboolean
gui_prefs_get_priority(void)
{
    if (appinfo->rcd_host_url)
        return appinfo->rcd_priority;

    return rcconfig.priority_enabled;
} /* gui_prefs_get_priority */

void
gui_prefs_set_priority(gboolean enabled)
{
    rcconfig.priority_enabled = enabled;

    gtk_toggle_button_set_active(
        GTK_TOGGLE_BUTTON(GET_WIDGET("priority_enable")), enabled);
} /* gui_prefs_set_priority */

void
gui_prefs_sync_priority(void)
{
    rc_config_set_bool("Priority/Enabled", rcconfig.priority_enabled);
    rc_config_sync();
} /* gui_prefs_sync_priority */    

/* This chunk is for the proxy info stuff that pops up when you first run
   the application. */
static void
apply_proxy_dialog_settings(void)
{
    ProxyType ptype;

    if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_none_rb"))))
        ptype = PROXY_TYPE_NONE;
    else if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_http_rb"))))
        ptype = PROXY_TYPE_HTTP;
    else if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_socks4_rb"))))
        ptype = PROXY_TYPE_SOCKS4;
    else if (gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_socks5_rb"))))
        ptype = PROXY_TYPE_SOCKS5;
    else {
        ptype = PROXY_TYPE_NONE;
        g_assert_not_reached();
    }
        
    rc_config_set_int("Proxy/Type", (int) ptype);
    rc_config_set_string(
        "Proxy/Host", gtk_entry_get_text(GTK_ENTRY(GET_WIDGET("dialog_host"))));
    rc_config_set_string(
        "Proxy/Port", gtk_entry_get_text(GTK_ENTRY(GET_WIDGET("dialog_port"))));
    rc_config_set_bool(
        "Proxy/UseAuth",
        gtk_toggle_button_get_active(
            GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_enable_auth"))));
    rc_config_set_string(
        "Proxy/Name", gtk_entry_get_text(
            GTK_ENTRY(GET_WIDGET("dialog_username"))));
    rc_config_private_set_string(
        "Proxy/Pass", gtk_entry_get_text(
            GTK_ENTRY(GET_WIDGET("dialog_password"))));

    rc_config_sync();
} /* apply_proxy_dialog_settings */

static void
proxy_auth_enabled(GtkToggleButton *button)
{
    gboolean proxy_active = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_enable_auth")));
    gboolean auth_active = gtk_toggle_button_get_active(button);

    gtk_widget_set_sensitive(
        GET_WIDGET("dialog_username_l"), proxy_active && auth_active);
    gtk_widget_set_sensitive(
        GET_WIDGET("dialog_password_l"), proxy_active && auth_active);
    gtk_widget_set_sensitive(
        GET_WIDGET("dialog_username"), proxy_active && auth_active);
    gtk_widget_set_sensitive(
        GET_WIDGET("dialog_password"), proxy_active && auth_active);
} /* proxy_auth_enabled */

static void
proxy_enabled(GtkToggleButton *button)
{
    gboolean active = !gtk_toggle_button_get_active(button);
    GtkToggleButton *auth = GTK_TOGGLE_BUTTON(GET_WIDGET("dialog_enable_auth"));

    gtk_widget_set_sensitive(GET_WIDGET("dialog_host_l"), active);
    gtk_widget_set_sensitive(GET_WIDGET("dialog_port_l"), active);
    gtk_widget_set_sensitive(GET_WIDGET("dialog_host"), active);
    gtk_widget_set_sensitive(GET_WIDGET("dialog_port"), active);
    gtk_widget_set_sensitive(GTK_WIDGET(auth), active);
    if (!active || (active && gtk_toggle_button_get_active(auth))) {
        gtk_widget_set_sensitive(GET_WIDGET("dialog_username_l"), active);
        gtk_widget_set_sensitive(GET_WIDGET("dialog_password_l"), active);
        gtk_widget_set_sensitive(GET_WIDGET("dialog_username"), active);
        gtk_widget_set_sensitive(GET_WIDGET("dialog_password"), active);
    }
} /* proxy_enabled */

void
rc_gui_proxy_dialog(void)
{
    GtkWidget *w;
    int button;

    w = GET_WIDGET("proxy_dialog");
    gnome_dialog_close_hides(GNOME_DIALOG(w), TRUE);
    gtk_signal_connect(
        GTK_OBJECT(GET_WIDGET("dialog_none_rb")), "toggled",
        GTK_SIGNAL_FUNC(proxy_enabled), NULL);
    gtk_signal_connect(
        GTK_OBJECT(GET_WIDGET("dialog_enable_auth")), "toggled",
        GTK_SIGNAL_FUNC(proxy_auth_enabled), NULL);

    button = gnome_dialog_run_and_close(GNOME_DIALOG(w));

    /* The user clicked on OK */
    if (button == 0)
        apply_proxy_dialog_settings();

    rc_config_set_bool("Proxy/ProxySet", TRUE);
    rc_config_sync();
} /* rc_gui_proxy_dialog */

void
rc_gui_maybe_proxy_dialog(void)
{
    gboolean done;

    done = rc_config_get_bool("Proxy/ProxySet=FALSE");
    if (!done)
        rc_gui_proxy_dialog();
} /* rc_gui_maybe_proxy_dialog */

static const char *
proxy_type_to_string(ProxyType proxy_type)
{
    const char *method;

    switch (proxy_type) {
    case PROXY_TYPE_HTTP:
        method = "http";
        break;
    case PROXY_TYPE_SOCKS4:
        method = "socks4";
        break;
    case PROXY_TYPE_SOCKS5:
        method = "socks5";
        break;
    default:
        g_warning("Unknown proxy type");
        method = NULL;
        break;
    }

    return method;
} /* proxy_type_to_string */

/* Functions for proactively applying settings to other parts of the code */
static void
apply_settings(void)
{
    gchar *url;
    GSList *transfers;
    GSList *i;
    gboolean priority;

    /* Set the proxy on all transfers */
    url = gui_prefs_get_proxy_url();

    transfers = gui_transfer_get_current_transfers();
    i = transfers;
    while (i) {
        GUITransfer *t = i->data;

        rc_debug (RC_DEBUG_LEVEL_INFO, "Setting proxy to: %s", url);
        gui_transfer_set_proxy_url(t, url);

        i = i->next;
    }

    g_free(url);

    /* Set the priority update mode */
    if (rc_gui_priority_is_initialized()) {
        priority = appinfo->priority;
        rc_gui_priority();
        if (priority != appinfo->priority)
            rc_gui_refresh_all_channels();
    }
} /* apply_settings */

void
gui_prefs_set_string (char *path, char *data)
{
    rc_config_set_string (path, data);
    rc_config_sync();
}

gchar *
gui_prefs_get_string (char *path)
{
    return rc_config_get_string (path);
}

/* some libglade callbacks */
void on_proxy_type_none_toggled (GtkWidget *toggle, gpointer data);
void on_proxy_type_other_toggled (GtkWidget *toggle, gpointer data);
void on_proxy_auth_toggled (GtkWidget *toggle, gpointer data);
void on_proxy_entry_changed (GtkWidget *w, GdkEvent *evt, gpointer data);

void on_cache_enable_toggled(GtkWidget *toggle, gpointer data);
void on_cache_path_changed(GtkWidget *w, GdkEvent *evt, gpointer data);
void on_cache_clear_packages_on_startup_toggled(GtkWidget *toggle, gpointer data);
void on_cache_clear_packages_on_exit_toggled(GtkWidget *toggle, gpointer data);
void on_cache_clear_packages_over_time_toggled(GtkWidget *toggle, gpointer data);
void on_cache_time_changed(GtkWidget *spinbutton, gpointer data);

void on_sign_ask_enable_toggled(GtkWidget *toggle, gpointer data);

void on_gui_show_pretty_names_toggled(GtkWidget *toggle, gpointer data);

void on_http10_enable_toggled(GtkWidget *toggle, gpointer data);

void on_priority_toggled(GtkWidget *toggle, gpointer data);

void on_property_dialog_apply (GtkWidget *dialog, gpointer data);
gboolean on_property_dialog_close (GtkWidget *dialog, gpointer data);

void on_clist_selected(GtkWidget *clist, int row, int column, GdkEvent *event);
void on_clist_unselected(GtkWidget *clist, int row, int col, GdkEvent *event);

static void
set_enabled_from_toggle (GtkWidget *toggle, gchar *widget)
{
    if (widget) {
        GtkWidget *w = GET_WIDGET (widget);
            
        gtk_widget_set_sensitive (w, GTK_TOGGLE_BUTTON (toggle)->active);
    }

    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
} /* set_enabled_from_toggle */

void
on_proxy_type_none_toggled (GtkWidget *toggle, gpointer data)
{
    GtkWidget *w = GET_WIDGET("proxy_table");

    if (GTK_TOGGLE_BUTTON(toggle)->active)
        gtk_widget_set_sensitive(w, FALSE);

    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
} /* on_proxy_type_none_toggled */

void
on_proxy_type_other_toggled (GtkWidget *toggle, gpointer data)
{
    GtkWidget *w = GET_WIDGET("proxy_table");

    if (GTK_TOGGLE_BUTTON(toggle)->active)
        gtk_widget_set_sensitive(w, TRUE);

    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
} /* on_proxy_type_other_toggled */

void
on_proxy_auth_toggled (GtkWidget *toggle, gpointer data)
{
    set_enabled_from_toggle (toggle, "proxy_name");
    set_enabled_from_toggle (toggle, "proxy_pass");
    set_enabled_from_toggle (toggle, "proxy_name_label");
    set_enabled_from_toggle (toggle, "proxy_pass_label");
}

void
on_proxy_entry_changed (GtkWidget *w, GdkEvent *evt, gpointer data)
{
    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
}

void
on_cache_path_changed(GtkWidget *w, GdkEvent *evt, gpointer data)
{
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));
}

void
on_cache_enable_toggled(GtkWidget *toggle, gpointer data)
{
    set_enabled_from_toggle(toggle, "cache_path_hbox");
}

void
on_cache_clear_packages_on_exit_toggled (GtkWidget *toggle, gpointer data)
{
    rcconfig.cache_clear_on_exit = GTK_TOGGLE_BUTTON (toggle)->active;
    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
}

void
on_cache_clear_packages_on_startup_toggled (GtkWidget *toggle, gpointer data)
{
    rcconfig.cache_clear_on_startup = GTK_TOGGLE_BUTTON (toggle)->active;
    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
}

void
on_cache_clear_packages_over_time_toggled (GtkWidget *toggle, gpointer data)
{
    rcconfig.cache_time_enabled = GTK_TOGGLE_BUTTON (toggle)->active;
    set_enabled_from_toggle(toggle, "cache_time_hbox");
}

void
on_cache_time_changed(GtkWidget *spinbutton, gpointer data)
{
    rcconfig.cache_time = gtk_editable_get_chars(
        GTK_EDITABLE(spinbutton), 0, -1);
    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
}

void
on_sign_ask_enable_toggled(GtkWidget *toggle, gpointer data)
{
    /* NULL is because we don't want to desensitize anything */
    set_enabled_from_toggle(toggle, NULL);

    if (!rcconfig.initializing)
        gnome_property_box_changed (GNOME_PROPERTY_BOX (rcconfig.dialog));
}

void
on_gui_show_pretty_names_toggled(GtkWidget *toggle, gpointer data)
{
    rcconfig.show_pretty_names = GTK_TOGGLE_BUTTON(toggle)->active;
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));
} /* on_gui_show_pretty_name_toggled */

void
on_http10_enable_toggled(GtkWidget *toggle, gpointer data)
{
    rcconfig.http10_enabled = GTK_TOGGLE_BUTTON(toggle)->active;
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));
} /* on_http10_enable_toggled */

void
on_priority_toggled(GtkWidget *toggle, gpointer data)
{
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));
} /* on_priority_toggled */

void
on_property_dialog_apply (GtkWidget *dialog, gpointer data)
{
    gui_prefs_save();
} /* on_property_dialog_apply */

gboolean
on_property_dialog_close (GtkWidget *dialog, gpointer data)
{
    return FALSE;
} /* on_property_dialog_close */

void
on_clist_selected(GtkWidget *clist, int row, int column, GdkEvent *event)
{
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));

    gtk_object_set_data(
        GTK_OBJECT(clist), "selected_row", GINT_TO_POINTER(row));
} /* on_clist_selected */

void
on_clist_unselected(GtkWidget *clist, int row, int column, GdkEvent *event)
{
    if (!rcconfig.initializing)
        gnome_property_box_changed(GNOME_PROPERTY_BOX(rcconfig.dialog));

    gtk_object_set_data(
        GTK_OBJECT(clist), "selected_row", GINT_TO_POINTER(-1));
} /* on_clist_unselected */


